@@ -1,6 +1,11 @@
 /*
- * Copyright (c) 1992, Brian Berliner and Jeff Polk
- * Copyright (c) 1989-1992, Brian Berliner
+ * Copyright (C) 1986-2005 The Free Software Foundation, Inc.
+ *
+ * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
+ *                                  and others.
+ *
+ * Portions Copyright (C) 1992, Brian Berliner and Jeff Polk
+ * Portions Copyright (C) 1989-1992, Brian Berliner
  * 
  * You may distribute under the terms of the GNU General Public License as
  * specified in the README file that comes with the CVS source distribution.
@@ -387,7 +392,7 @@ Classify_File (struct file_info *finfo, char *tag, char *date, char *options,
     }
 
     /* free up the vers struct, or just return it */
-    if (versp != (Vers_TS **) NULL)
+    if (versp != NULL)
 	*versp = vers;
     else
 	freevers_ts (&vers);
