@@ -15,8 +15,6 @@
 #include "cvs.h"
 #include "getline.h"
 
-#ifdef AUTH_CLIENT_SUPPORT   /* This covers the rest of the file. */
-
 /* There seems to be very little agreement on which system header
    getpass is declared in.  With a lot of fancy autoconfiscation,
    we could perhaps detect this, but for now we'll just rely on
@@ -24,6 +22,10 @@
    declaration won't work (some Crays declare the 2#$@% thing as
    varadic, believe it or not).  On Cray, getpass will be declared
    in either stdlib.h or unistd.h.  */
+#include "getpass.h"
+
+#ifdef AUTH_CLIENT_SUPPORT   /* This covers the rest of the file. */
+
 
 #ifndef CVS_PASSWORD_FILE 
 #define CVS_PASSWORD_FILE ".cvspass"
@@ -32,11 +34,11 @@
 /* If non-NULL, get_cvs_password() will just return this. */
 static char *cvs_password = NULL;
 
-static char *construct_cvspass_filename PROTO ((void));
+static char *construct_cvspass_filename (void);
 
 /* The return value will need to be freed. */
 static char *
-construct_cvspass_filename ()
+construct_cvspass_filename (void)
 {
     char *homedir;
     char *passfile;
@@ -58,7 +60,7 @@ construct_cvspass_filename ()
 	   right thing for a GUI is to just store the password in
 	   memory only)...  */
 	error (1, 0, "could not find out home directory");
-	return (char *) NULL;
+	return NULL;
     }
 
     passfile = strcat_filename_onto_homedir (homedir, CVS_PASSWORD_FILE);
@@ -101,11 +103,9 @@ construct_cvspass_filename ()
  *	called on the same linebuf
  */
 static char *
-password_entry_parseline (cvsroot_canonical, warn, linenumber, linebuf)
-    const char *cvsroot_canonical;
-    const unsigned char warn;
-    const int linenumber;
-    char *linebuf;
+password_entry_parseline (const char *cvsroot_canonical,
+			  const unsigned char warn, const int linenumber,
+			  char *linebuf)
 {
     char *password = NULL;
     char *p;
@@ -205,7 +205,6 @@ password_entry_parseline (cvsroot_canonical, warn, linenumber, linebuf)
 	    password = p + 1;
 
 	free (tmp_root_canonical);
-	free_cvsroot_t (tmp_root);
     }
 
     return password;
@@ -279,10 +278,7 @@ typedef enum password_entry_operation_e {
 } password_entry_operation_t;
 
 static char *
-password_entry_operation (operation, root, newpassword)
-    password_entry_operation_t operation;
-    cvsroot_t *root;
-    char *newpassword;
+password_entry_operation (password_entry_operation_t operation, cvsroot_t *root, char *newpassword)
 {
     char *passfile;
     FILE *fp;
@@ -318,7 +314,7 @@ internal error: can only call password_entry_operation with pserver method");
     }
 
     /* Check each line to see if we have this entry already. */
-    line = 0;
+    line = 0L;
     while ((line_length = getline (&linebuf, &linebuf_len, fp)) >= 0)
     {
 	line++;
@@ -387,10 +383,9 @@ process:
 
 	/* create and open a temp file */
 	if ((tmp_fp = cvs_temp_file (&tmp_name)) == NULL)
-	    error (1, errno, "unable to open temp file %s",
-		   tmp_name ? tmp_name : "(null)");
+	    error (1, errno, "unable to open temp file %s", tmp_name);
 
-	line = 0;
+	line = 0L;
 	while ((line_length = getline (&linebuf, &linebuf_len, fp)) >= 0)
 	{
 	    line++;
@@ -517,9 +512,7 @@ static const char *const login_usage[] =
 };
 
 int
-login (argc, argv)
-    int argc;
-    char **argv;
+login (int argc, char **argv)
 {
     char *typed_password;
     char *cvsroot_canonical;
@@ -578,12 +571,11 @@ login (argc, argv)
 
 
 
-/* Returns the _scrambled_ password in freshly allocated memory.  The server
- * must descramble before hashing and comparing.  If password file not found,
- * or password not found in the file, just return NULL.
- */
+/* Returns the _scrambled_ password.  The server must descramble
+   before hashing and comparing.  If password file not found, or
+   password not found in the file, just return NULL. */
 char *
-get_cvs_password ()
+get_cvs_password (void)
 {
     if (current_parsed_root->password)
 	return scramble (current_parsed_root->password);
@@ -592,7 +584,7 @@ get_cvs_password ()
        context, then assume they have supplied the correct, scrambled
        password. */
     if (cvs_password)
-	return xstrdup (cvs_password);
+	return cvs_password;
 
     if (getenv ("CVS_PASSWORD") != NULL)
     {
@@ -629,9 +621,7 @@ static const char *const logout_usage[] =
 
 /* Remove any entry for the CVSRoot repository found in .cvspass. */
 int
-logout (argc, argv)
-    int argc;
-    char **argv;
+logout (int argc, char **argv)
 {
     char *cvsroot_canonical;
 
