@@ -1,6 +1,11 @@
 /*
- *    Copyright (c) 1992, Brian Berliner and Jeff Polk
- *    Copyright (c) 1989-1992, Brian Berliner
+ * Copyright (C) 1986-2005 The Free Software Foundation, Inc.
+ *
+ * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
+ *                                  and others.
+ *
+ * Portions Copyright (C) 1992, Brian Berliner and Jeff Polk
+ * Portions Copyright (C) 1989-1992, Brian Berliner
  *
  *    You may distribute under the terms of the GNU General Public License
  *    as specified in the README file that comes with the CVS source
@@ -70,11 +75,8 @@ open_module (void)
 	error (0, 0, "must set the CVSROOT environment variable");
 	error (1, 0, "or specify the '-d' global option");
     }
-    mfile = xmalloc (strlen (current_parsed_root->directory)
-		     + sizeof (CVSROOTADM)
-		     + sizeof (CVSROOTADM_MODULES) + 3);
-    (void) sprintf (mfile, "%s/%s/%s", current_parsed_root->directory,
-		    CVSROOTADM, CVSROOTADM_MODULES);
+    mfile = Xasprintf ("%s/%s/%s", current_parsed_root->directory,
+		       CVSROOTADM, CVSROOTADM_MODULES);
     retval = dbm_open (mfile, O_RDONLY, 0666);
     free (mfile);
     return retval;
@@ -239,7 +241,7 @@ my_module (DBM *db, char *mname, enum mtype m_type, char *msg,
 		/* if mname was a file, we have to split it into "dir file" */
 		if ((cp = strrchr (mname, '/')) != NULL && cp != mname)
 		{
-		    modargv = xmalloc (2 * sizeof (*modargv));
+		    modargv = xnmalloc (2, sizeof (*modargv));
 		    modargv[0] = xmalloc (strlen (mname) + 2);
 		    strncpy (modargv[0], mname, cp - mname);
 		    modargv[0][cp - mname] = '\0';
@@ -256,7 +258,7 @@ my_module (DBM *db, char *mname, enum mtype m_type, char *msg,
 		    if (cp == mname)
 		    {
 			/* drop the leading / if specified */
-			modargv = xmalloc (2 * sizeof (*modargv));
+			modargv = xnmalloc (2, sizeof (*modargv));
 			modargv[0] = xstrdup (".");
 			modargv[1] = xstrdup (mname + 1);
 			modargc = 2;
@@ -264,7 +266,7 @@ my_module (DBM *db, char *mname, enum mtype m_type, char *msg,
 		    else
 		    {
 			/* otherwise just copy it */
-			modargv = xmalloc (2 * sizeof (*modargv));
+			modargv = xnmalloc (2, sizeof (*modargv));
 			modargv[0] = xstrdup (".");
 			modargv[1] = xstrdup (mname);
 			modargc = 2;
@@ -344,6 +346,7 @@ my_module (DBM *db, char *mname, enum mtype m_type, char *msg,
 	    /* mwhere gets just the module name */
 	    mwhere = xstrdup (mname);
 	    mfile = cp + 1;
+	    assert (strlen (mfile));
 
 	    /* put the / back in mname */
 	    *cp = '/';
@@ -399,9 +402,7 @@ my_module (DBM *db, char *mname, enum mtype m_type, char *msg,
      */
 
     /* Put the value on a line with XXX prepended for getopt to eat */
-    line = xmalloc (strlen (value) + 5);
-    strcpy(line, "XXX ");
-    strcpy(line + 4, value);
+    line = Xasprintf ("XXX %s", value);
 
     /* turn the line into an argv[] array */
     line2argv (&xmodargc, &xmodargv, line, " \t");
@@ -556,13 +557,12 @@ module `%s' is a request for a file in a module which is not a directory",
 
 	    nullrepos = emptydir_name ();
 
-	    Create_Admin (".", dir,
-			  nullrepos, (char *) NULL, (char *) NULL, 0, 0, 1);
+	    Create_Admin (".", dir, nullrepos, NULL, NULL, 0, 0, 1);
 	    if (!noexec)
 	    {
 		FILE *fp;
 
-		fp = open_file (CVSADM_ENTSTAT, "w+");
+		fp = xfopen (CVSADM_ENTSTAT, "w+");
 		if (fclose (fp) == EOF)
 		    error (1, errno, "cannot close %s", CVSADM_ENTSTAT);
 #ifdef SERVER_SUPPORT
@@ -603,19 +603,10 @@ module `%s' is a request for a file in a module which is not a directory",
 	change_to = where ? where : (mwhere ? mwhere : mname);
 	server_dir_to_restore = server_dir;
 	restore_server_dir = 1;
-	server_dir =
-	    xmalloc ((server_dir_to_restore != NULL
-		      ? strlen (server_dir_to_restore)
-		      : 0)
-		     + strlen (change_to)
-		     + 5);
-	server_dir[0] = '\0';
 	if (server_dir_to_restore != NULL)
-	{
-	    strcat (server_dir, server_dir_to_restore);
-	    strcat (server_dir, "/");
-	}
-	strcat (server_dir, change_to);
+	    server_dir = Xasprintf ("%s/%s", server_dir_to_restore, change_to);
+	else
+	    server_dir = xstrdup (change_to);
     }
 #endif
 
@@ -646,9 +637,8 @@ module `%s' is a request for a file in a module which is not a directory",
 		   CVSMODULE_SPEC, mname);
 	else
 	    err += my_module (db, spec_opt, m_type, msg, callback_proc,
-                               (char *) NULL, 0, local_specified,
-                               run_module_prog, build_dirs, extra_arg,
-	                       stack);
+			      NULL, 0, local_specified, run_module_prog,
+			      build_dirs, extra_arg, stack);
 	spec_opt = next_opt;
     }
 
@@ -688,9 +678,7 @@ module `%s' is a request for a file in a module which is not a directory",
 
 	    if ((*prog != '/') && (*prog != '.'))
 	    {
-		real_prog = xmalloc (strlen (real_where) + strlen (prog)
-				     + 10);
-		(void) sprintf (real_prog, "%s/%s", real_where, prog);
+		real_prog = Xasprintf ("%s/%s", real_where, prog);
 		if (isfile (real_prog))
 		    prog = real_prog;
 	    }
@@ -700,10 +688,10 @@ module `%s' is a request for a file in a module which is not a directory",
 	    if (expanded_path != NULL)
 	    {
 		run_setup (expanded_path);
-		run_arg (real_where);
+		run_add_arg (real_where);
 
 		if (extra_arg)
-		    run_arg (extra_arg);
+		    run_add_arg (extra_arg);
 
 		if (!quiet)
 		{
@@ -718,7 +706,7 @@ module `%s' is a request for a file in a module which is not a directory",
 		err += run_exec (RUN_TTY, RUN_TTY, RUN_TTY, RUN_NORMAL);
 		free (expanded_path);
 	    }
-	    free (real_prog);
+	    if (real_prog) free (real_prog);
 	}
     }
 
@@ -830,7 +818,7 @@ save_d (char *k, int ks, char *d, int ds)
     if (s_count == s_max)
     {
 	s_max += 64;
-	s_head = (struct sortrec *) xrealloc ((char *) s_head, s_max * sizeof (*s_head));
+	s_head = xnrealloc (s_head, s_max, sizeof (*s_head));
     }
     s_rec = &s_head[s_count];
     s_rec->modname = cp = xmalloc (ks + 1);
@@ -941,21 +929,18 @@ cat_module (int status)
 	char *line;
 
 	/* Print module name (and status, if wanted) */
-	line = xmalloc (strlen (s_h->modname) + 15);
-	sprintf (line, "%-12s", s_h->modname);
+	line = Xasprintf ("%-12s", s_h->modname);
 	cvs_output (line, 0);
 	free (line);
 	if (status)
 	{
-	    line = xmalloc (strlen (s_h->status) + 15);
-	    sprintf (line, " %-11s", s_h->status);
+	    line = Xasprintf (" %-11s", s_h->status);
 	    cvs_output (line, 0);
 	    free (line);
 	}
 
-	line = xmalloc (strlen (s_h->modname) + strlen (s_h->rest) + 15);
 	/* Parse module file entry as command line and print options */
-	(void) sprintf (line, "%s %s", s_h->modname, s_h->rest);
+	line = Xasprintf ("%s %s", s_h->modname, s_h->rest);
 	line2argv (&moduleargc, &moduleargv, line, " \t");
 	free (line);
 	argc = moduleargc;
