@@ -1,6 +1,11 @@
 /*
- * Copyright (c) 1992, Brian Berliner and Jeff Polk
- * Copyright (c) 1989-1992, Brian Berliner
+ * Copyright (C) 1986-2005 The Free Software Foundation, Inc.
+ *
+ * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
+ *                                  and others.
+ *
+ * Portions Copyright (C) 1992, Brian Berliner and Jeff Polk
+ * Portions Copyright (C) 1989-1992, Brian Berliner
  * 
  * You may distribute under the terms of the GNU General Public License as
  * specified in the README file that comes with the CVS source distribution.
@@ -96,7 +101,7 @@ call_diff_setup (const char *prog)
 	if (call_diff_argv[i])
 	{
 	    free (call_diff_argv[i]);
-	    call_diff_argv[i] = (char *) 0;
+	    call_diff_argv[i] = NULL;
 	}
     }
     call_diff_argc = 0;
@@ -106,7 +111,7 @@ call_diff_setup (const char *prog)
     /* put each word into call_diff_argv, allocating it as we go */
     for (cp = strtok (call_diff_prog, " \t");
 	 cp != NULL;
-	 cp = strtok ((char *) NULL, " \t"))
+	 cp = strtok (NULL, " \t"))
 	call_diff_add_arg (cp);
     free (call_diff_prog);
 }
@@ -124,16 +129,16 @@ call_diff_add_arg (const char *s)
     if (call_diff_argc >= call_diff_argc_allocated)
     {
 	call_diff_argc_allocated += 50;
-	call_diff_argv = (char **)
-	    xrealloc ((char *) call_diff_argv,
-		      call_diff_argc_allocated * sizeof (char **));
+	call_diff_argv = xnrealloc (call_diff_argv,
+				    call_diff_argc_allocated,
+				    sizeof (char **));
     }
 
     if (s)
 	call_diff_argv[call_diff_argc++] = xstrdup (s);
     else
 	/* Not post-incremented on purpose!  */
-	call_diff_argv[call_diff_argc] = (char *) 0;
+	call_diff_argv[call_diff_argc] = NULL;
 }
 
 /* Callback function for the diff library to write data to the output
@@ -189,8 +194,8 @@ static struct diff_callbacks call_diff_stdout_callbacks =
 
 static struct diff_callbacks call_diff_file_callbacks =
 {
-    (void (*) (const char *, size_t)) NULL,
-    (void (*) (void)) NULL,
+    NULL,
+    NULL,
     call_diff_write_stdout,
     call_diff_error
 };
@@ -247,6 +252,7 @@ RCS_merge (RCSNode *rcs, const char *path, const char *workfile,
        numeric revisions.  */
     xrev1 = RCS_gettag (rcs, rev1, 0, NULL);
     xrev2 = RCS_gettag (rcs, rev2, 0, NULL);
+    assert (xrev1 && xrev2);
 
     /* Check out chosen revisions.  The error message when RCS_checkout
        fails is not very informative -- it is taken verbatim from RCS 5.7,
@@ -256,8 +262,7 @@ RCS_merge (RCSNode *rcs, const char *path, const char *workfile,
     cvs_output ("\n", 1);
 
     tmp1 = cvs_temp_name();
-    if (RCS_checkout (rcs, NULL, xrev1, rev1, options, tmp1,
-		      (RCSCHECKOUTPROC)0, NULL))
+    if (RCS_checkout (rcs, NULL, xrev1, rev1, options, tmp1, NULL, NULL))
     {
 	cvs_outerr ("rcsmerge: co failed\n", 0);
 	exit (EXIT_FAILURE);
@@ -268,8 +273,7 @@ RCS_merge (RCSNode *rcs, const char *path, const char *workfile,
     cvs_output ("\n", 1);
 
     tmp2 = cvs_temp_name();
-    if (RCS_checkout (rcs, NULL, xrev2, rev2, options, tmp2,
-		      (RCSCHECKOUTPROC)0, NULL))
+    if (RCS_checkout (rcs, NULL, xrev2, rev2, options, tmp2, NULL, NULL))
     {
 	cvs_outerr ("rcsmerge: co failed\n", 0);
 	exit (EXIT_FAILURE);
@@ -399,7 +403,7 @@ RCS file: ", 0);
     {
 	tmpfile1 = cvs_temp_name();
 	status = RCS_checkout (rcsfile, NULL, rev1, NULL, options, tmpfile1,
-	                       (RCSCHECKOUTPROC)0, NULL);
+	                       NULL, NULL);
 	if (status > 0)
 	{
 	    retval = status;
@@ -427,7 +431,7 @@ RCS file: ", 0);
 	cvs_output (rev2, 0);
 	cvs_output ("\n", 1);
 	status = RCS_checkout (rcsfile, NULL, rev2, NULL, options,
-			       tmpfile2, (RCSCHECKOUTPROC)0, NULL);
+			       tmpfile2, NULL, NULL);
 	if (status > 0)
 	{
 	    retval = status;
@@ -560,9 +564,8 @@ diff_exec (const char *file1, const char *file2, const char *label1,
     }
 #endif
 
-    args = xmalloc (strlen (options) + 10);
     /* The first word in this string is used only for error reporting. */
-    sprintf (args, "diff %s", options);
+    args = Xasprintf ("diff %s", options);
     call_diff_setup (args);
     if (label1)
 	call_diff_arg (label1);
@@ -588,9 +591,7 @@ RCS_output_diff_options (const char *opts, const char *rev1, const char *rev2,
 {
     char *tmp;
 
-    tmp = (char *) xmalloc (strlen (opts) + strlen (rev1) + 10);
-
-    sprintf (tmp, "diff%s -r%s", opts, rev1);
+    tmp = Xasprintf ("diff%s -r%s", opts, rev1);
     cvs_output (tmp, 0);
     free (tmp);
 
