@@ -99,13 +99,9 @@ void wrap_setup(void)
     {
 	char *file;
 
-	file = xmalloc (strlen (current_parsed_root->directory)
-			+ sizeof (CVSROOTADM)
-			+ sizeof (CVSROOTADM_WRAPPER)
-			+ 3);
 	/* Then add entries found in repository, if it exists.  */
-	(void) sprintf (file, "%s/%s/%s", current_parsed_root->directory, CVSROOTADM,
-			CVSROOTADM_WRAPPER);
+	file = Xasprintf ("%s/%s/%s", current_parsed_root->directory,
+			  CVSROOTADM, CVSROOTADM_WRAPPER);
 	if (isfile (file))
 	{
 	    wrap_add_file(file,0);
@@ -214,22 +210,10 @@ wrap_unparse_rcs_options (char **line, int first_call_p)
         return;
     }
 
-    *line = xmalloc (strlen (wrap_list[i]->wildCard)
-                     + strlen ("\t")
-                     + strlen (" -k '")
-                     + (wrap_list[i]->rcsOption != NULL ? 
-                           strlen (wrap_list[i]->rcsOption) : 2)
-                     + strlen ("'")
-                     + 1);  /* leave room for '\0' */
-
-    strcpy (*line, wrap_list[i]->wildCard);
-    strcat (*line, " -k '");
-    if (wrap_list[i]->rcsOption != NULL)
-        strcat (*line, wrap_list[i]->rcsOption);
-    else
-        strcat (*line, "kv");
-    strcat (*line, "'");
-
+    *line = Xasprintf ("%s -k '%s'",
+		       wrap_list[i]->wildCard,
+		       wrap_list[i]->rcsOption
+		       ? wrap_list[i]->rcsOption : "kv");
     ++i;
 }
 #endif /* SERVER_SUPPORT || CLIENT_SUPPORT */
@@ -459,24 +443,24 @@ wrap_add (char *line, int isTemp)
 }
 
 void
-wrap_add_entry(WrapperEntry *e, int temp)
+wrap_add_entry (WrapperEntry *e, int temp)
 {
     int x;
-    if(wrap_count+wrap_tempcount>=wrap_size){
+    if (wrap_count + wrap_tempcount >= wrap_size)
+    {
 	wrap_size += WRAPPER_GROW;
-	wrap_list = (WrapperEntry **) xrealloc ((char *) wrap_list,
-						wrap_size *
-						sizeof (WrapperEntry *));
+	wrap_list = xnrealloc (wrap_list, wrap_size, sizeof (WrapperEntry *));
     }
 
-    if(!temp && wrap_tempcount){
-	for(x=wrap_count+wrap_tempcount-1;x>=wrap_count;--x)
-	    wrap_list[x+1]=wrap_list[x];
+    if (!temp && wrap_tempcount)
+    {
+	for (x = wrap_count + wrap_tempcount - 1; x >= wrap_count; --x)
+	    wrap_list[x + 1] = wrap_list[x];
     }
 
-    x=(temp ? wrap_count+(wrap_tempcount++):(wrap_count++));
-    wrap_list[x]=(WrapperEntry *)xmalloc(sizeof(WrapperEntry));
-    *wrap_list[x]=*e;
+    x = (temp ? wrap_count + (wrap_tempcount++) : (wrap_count++));
+    wrap_list[x] = xmalloc (sizeof (WrapperEntry));
+    *wrap_list[x] = *e;
 }
 
 /* Return 1 if the given filename is a wrapper filename */
@@ -519,7 +503,7 @@ wrap_matching_entry (const char *name)
     for(x=0;x<count;++x)
 	if (CVS_FNMATCH (wrap_list[x]->wildCard, name, 0) == 0)
 	    return wrap_list[x];
-    return (WrapperEntry *)NULL;
+    return NULL;
 }
 
 /* Return the RCS options for FILENAME in a newly malloc'd string.  If
@@ -529,22 +513,11 @@ char *
 wrap_rcsoption (const char *filename, int asflag)
 {
     WrapperEntry *e = wrap_matching_entry (filename);
-    char *buf;
 
     if (e == NULL || e->rcsOption == NULL || (*e->rcsOption == '\0'))
 	return NULL;
 
-    buf = xmalloc (strlen (e->rcsOption) + 3);
-    if (asflag)
-    {
-	strcpy (buf, "-k");
-	strcat (buf, e->rcsOption);
-    }
-    else
-    {
-	strcpy (buf, e->rcsOption);
-    }
-    return buf;
+    return Xasprintf ("%s%s", asflag ? "-k" : "", e->rcsOption);
 }
 
 char *
@@ -561,14 +534,10 @@ wrap_tocvs_process_file(const char *fileName)
 	free (buf);
     buf = cvs_temp_name ();
 
-    args = xmalloc (strlen (e->tocvsFilter)
-		    + strlen (fileName)
-		    + strlen (buf));
-
-    wrap_clean_fmt_str(e->tocvsFilter, 2);
-    sprintf (args, e->tocvsFilter, fileName, buf);
+    wrap_clean_fmt_str (e->tocvsFilter, 2);
+    args = Xasprintf (e->tocvsFilter, fileName, buf);
     run_setup (args);
-    run_exec(RUN_TTY, RUN_TTY, RUN_TTY, RUN_NORMAL|RUN_REALLY );
+    run_exec (RUN_TTY, RUN_TTY, RUN_TTY, RUN_NORMAL | RUN_REALLY );
     free (args);
 
     return buf;
@@ -588,18 +557,15 @@ void
 wrap_fromcvs_process_file(const char *fileName)
 {
     char *args;
-    WrapperEntry *e=wrap_matching_entry(fileName);
+    WrapperEntry *e = wrap_matching_entry(fileName);
 
-    if(e==NULL || e->fromcvsFilter==NULL)
-	return;
-
-    args = xmalloc (strlen (e->fromcvsFilter)
-		    + strlen (fileName));
-
-    wrap_clean_fmt_str(e->fromcvsFilter, 1);
-    sprintf (args, e->fromcvsFilter, fileName);
-    run_setup (args);
-    run_exec(RUN_TTY, RUN_TTY, RUN_TTY, RUN_NORMAL );
-    free (args);
+    if (e != NULL && e->fromcvsFilter != NULL)
+    {
+	wrap_clean_fmt_str (e->fromcvsFilter, 1);
+	args = Xasprintf (e->fromcvsFilter, fileName);
+	run_setup (args);
+	run_exec (RUN_TTY, RUN_TTY, RUN_TTY, RUN_NORMAL);
+	free (args);
+    }
     return;
 }
