@@ -22,10 +22,7 @@
 #include <io.h>
 #include <fcntl.h>
 
-static void run_add_arg PROTO((const char *s));
-static void run_init_prog PROTO((void));
 
-//extern char *strtok ();
 
 /*
  * To exec a program under CVS, first call run_setup() to setup any initial
@@ -44,24 +41,23 @@ static char **run_argv;
 static int run_argc;
 static int run_argc_allocated;
 
-
-
-void
-run_arg_free_p (int argc, char **argv)
-{
-    int i;
-    for (i = 0; i < argc; i++)
-	free (argv[i]);
-}
-
 void
 run_setup (const char *prog)
 {
     char *cp;
+    int i;
+
     char *run_prog;
 
     /* clean out any malloc'ed values from run_argv */
-    run_arg_free_p (run_argc, run_argv);
+    for (i = 0; i < run_argc; i++)
+    {
+	if (run_argv[i])
+	{
+	    free (run_argv[i]);
+	    run_argv[i] = (char *) 0;
+	}
+    }
     run_argc = 0;
 
     run_prog = xstrdup (prog);
@@ -73,35 +69,8 @@ run_setup (const char *prog)
     free (run_prog);
 }
 
-void
-run_arg (s)
-    const char *s;
-{
-    run_add_arg (s);
-}
 
 
-
-void
-run_add_arg_p (iargc, iarg_allocated, iargv, s)
-    int *iargc;
-    size_t *iarg_allocated;
-    char ***iargv;
-    const char *s;
-{
-    /* allocate more argv entries if we've run out */
-    if (*iargc >= *iarg_allocated)
-    {
-	*iarg_allocated += 50;
-	*iargv = xrealloc (*iargv, *iarg_allocated * sizeof (char **));
-    }
-
-    if (s)
-	(*iargv)[(*iargc)++] = xstrdup (s);
-    else
-	(*iargv)[*iargc] = NULL;	/* not post-incremented on purpose! */
-}
-
 /* Return a malloc'd copy of s, with double quotes around it.  */
 static char *
 quote (const char *s)
@@ -131,7 +100,9 @@ quote (const char *s)
     return copy;
 }
 
-static void
+
+
+void
 run_add_arg (s)
     const char *s;
 {
@@ -139,8 +110,7 @@ run_add_arg (s)
     if (run_argc >= run_argc_allocated)
     {
 	run_argc_allocated += 50;
-	run_argv = (char **) xrealloc ((char *) run_argv,
-				     run_argc_allocated * sizeof (char **));
+	run_argv = xrealloc (run_argv, run_argc_allocated * sizeof (char **));
     }
 
     if (s)
@@ -149,9 +119,11 @@ run_add_arg (s)
 	run_argc++;
     }
     else
-	run_argv[run_argc] = (char *) 0;	/* not post-incremented on purpose! */
+	run_argv[run_argc] = NULL;	/* not post-incremented on purpose!  */
 }
 
+
+
 int
 run_exec (stin, stout, sterr, flags)
     const char *stin;
@@ -547,7 +519,7 @@ start_child (char *command, HANDLE in, HANDLE out)
    construct a command line that one might pass to CreateProcess.
    Try to quote things appropriately.  */
 static char *
-build_command (char **argv)
+build_command (char *const *argv)
 {
     int len;
 
@@ -620,7 +592,7 @@ build_command (char **argv)
    Return the handle of the child process (this is what
    _cwait and waitpid expect).  */
 int
-piped_child (const char **argv, int *to, int *from, int fix_stderr)
+piped_child (char *const *argv, int *to, int *from)
 {
   int child;
   HANDLE pipein[2], pipeout[2];
@@ -628,7 +600,7 @@ piped_child (const char **argv, int *to, int *from, int fix_stderr)
 
   /* Turn argv into a form acceptable to CreateProcess.  */
   command = build_command (argv);
-  if (! command)
+  if (!command)
       return -1;
 
   /* Create pipes for communicating with child.  Arrange for
@@ -720,6 +692,16 @@ filter_stream_through_program (oldfd, dir, prog, pidp)
 }
 
 
+
+int
+run_piped (int *tofdp, int *fromfdp)
+{
+    run_add_arg (NULL);
+    return piped_child (run_argv, tofdp, fromfdp);
+}
+
+
+
 /* Arrange for the file descriptor FD to not be inherited by child
    processes.  At the moment, CVS uses this function only on pipes
    returned by piped_child, and our implementation of piped_child
